/**
 * CG3 Billiards Assignment
 * Jason Winnebeck
 * April 23, 2003
 */

#include "global.h"
#include "StaticModel.h"
#include "GameException.h"

using namespace std;

std::string StaticModel::modelDir( "" );
LPD3DXMESH StaticModel::debugMesh = NULL;

StaticModel::StaticModel( const std::string& modelName ) {
    LPD3DXBUFFER mats;
    HRESULT r = 0;

    string fileName = modelDir + modelName;

    r = D3DXLoadMeshFromX(
        const_cast<LPSTR>(fileName.c_str()),
        D3DXMESH_MANAGED,
        g_dx->getDeviceHandle(),
        NULL,
        &mats,
        (DWORD*)&numMats,
        &mesh );

    if ( FAILED( r ) )
        throw GameException( "Could not load model " + fileName + ".", r );

    //The mesh is loaded, let's load the materials:
    materials = new D3DMATERIAL8[ numMats ];
    textures = new Texture[ numMats ];

    D3DXMATERIAL* modelMats = (D3DXMATERIAL*)mats->GetBufferPointer();

    for ( int i = 0; i < numMats; ++i ) {
        materials[ i ] = modelMats[ i ].MatD3D;
        materials[ i ].Ambient = materials[ i ].Diffuse;

        if ( modelMats[i].pTextureFilename ) {
            //Attempt to load this texture
            //Extract just the filename part.
            string tfileName( modelMats[i].pTextureFilename );
            string::size_type pos = tfileName.find_last_of( "\\/" );
            if ( pos != string::npos ) {
                tfileName = string( tfileName, pos+1 );
            }

            //Add the directory
            tfileName = modelDir + tfileName;

            //Attempt to load
            r = D3DXCreateTextureFromFile(
                g_dx->getDeviceHandle(),
                tfileName.c_str(),
                &textures[ i ] );

            if ( FAILED( r ) ) {
                //cleanup
                mats->Release();
                delete[] materials;
                for ( int c = 0; c < i; ++c )
                    textures[c]->Release();
                delete[] textures;
                mesh->Release();

                throw GameException( "Could not load texture " + tfileName + " for model " + modelName + ".", r );
            }

        } else {
            textures[i] = NULL;
        }
    }

    mats->Release();
}

StaticModel::~StaticModel() {
    mesh->Release();
    delete[] materials;

    for ( int i = 0; i < numMats; ++i ) {
        if ( textures[ i ] )
            textures[ i ]->Release();
    }

    delete[] textures;
}

void StaticModel::render() {
    HRESULT r = 0;
    GraphicsDevice dev = g_dx->getDeviceHandle();

    for ( int i = 0; i < numMats; ++i ) {
        r = dev->SetMaterial( &materials[i] );
        assert( SUCCEEDED( r ) );

        r = dev->SetTexture( 0, textures[i] );
        assert( SUCCEEDED( r ) );

        r = mesh->DrawSubset( (DWORD)i );
        assert( SUCCEEDED( r ) );
    }

    if ( debugMesh ) {
        //g_dx->RestoreDefaultMaterial();
        r = debugMesh->DrawSubset( 0 );
        assert( SUCCEEDED( r ) );
    }
}

void StaticModel::setModelDirectory( const std::string& newDir ) {
    modelDir = newDir;
}

void StaticModel::initDebugMesh() {
    HRESULT r = D3DXCreateSphere(
        g_dx->getDeviceHandle(),
        0.1f,
        20,
        20,
        &debugMesh,
        NULL );

    if ( FAILED( r ) )
        throw GameException( "Error creating debug mesh.", r );
}
