/**
 * CG3 Billiards Assignment
 * Jason Winnebeck
 * April 23, 2003
 */

#ifndef STATICMODEL_H_IMF423
#define STATICMODEL_H_IMF423

#include "global.h"

/**
 * The StaticModel class represents a static model that has no sort of
 * animation.
 *
 * Static models consist of a geometry mesh, and a set of 1 or more
 * materials and textures to apply to that mesh.
 *
 * The StaticModel class loads its models from the directory set in the static
 * member "modelDirectory", and attempts to load any textures in that same
 * directory.  Pathnames to files in the .x files are ignored and instead the
 * files are searched for in the "modelDirectory" directory.
 */
class StaticModel {
public:
    /**
     * Loads a StaticModel from the given "X" file in the directory given by
     * setModelDirectory.
     *
     * If the file is not found, or the model cannot be loaded for some other
     * reason (such as texture not found), a GameException is thrown.
     */
    explicit StaticModel( const std::string& modelName );

    /**
     * Destructor.
     */
    virtual ~StaticModel();

    /**
     * Renders the model.  The appropriate world matrix should be set if
     * desired.  Vertex shaders, textures, materials, and stream sources may
     * be set to different values, and not restored.
     */
    void render();

    /**
     * Sets a new directory to look for models and textures.  This only affects
     * models created after this method call.
     */
    static void setModelDirectory( const std::string& newDir );

    /**
     * Initializes the debug mesh, so that it draws a sphere around every model's
     * origin.
     */
    static void initDebugMesh();

private:
    static std::string modelDir;
    static LPD3DXMESH debugMesh;

    StaticModel();
    StaticModel( const StaticModel& o );
    StaticModel& operator = ( const StaticModel& rhs );

    LPD3DXMESH mesh;

    int numMats;
    D3DMATERIAL8* materials;
    Texture* textures;
};

#endif
