/**
 * CG3 Skeletal Animation Project
 * Jason Winnebeck
 * May 12, 2003
 */

#include "global.h"
#include "GameException.h"

const int DEFAULT_WIN_WIDTH = 800;
const int DEFAULT_WIN_HEIGHT = 600;

Window::Window() : width( DEFAULT_WIN_WIDTH ), height( DEFAULT_WIN_HEIGHT ) {
    TRACE("Created Game Window Object");
}

Window::~Window() {
    CoUninitialize(); //to clean up windows a little better. not required.
    TRACE("Terminating Game Window Object");
    //nothing we are killing this structure...
}

void Window::registerClass(HINSTANCE hInstance, const char* ATitle) {
    WNDCLASSEX wcex;
    wcex.cbSize = sizeof(WNDCLASSEX);

    wcex.style = CS_HREDRAW | CS_VREDRAW;
    wcex.lpfnWndProc = (WNDPROC)GlobalWndProc;
    wcex.cbClsExtra = 0;
    wcex.cbWndExtra = 0;
    wcex.hInstance = hInstance;
    wcex.hIcon = LoadIcon( NULL, IDI_APPLICATION );
    wcex.hIconSm = LoadIcon( NULL, IDI_HAND );
    wcex.hCursor = LoadCursor( NULL, IDC_ARROW );
    wcex.hbrBackground = (HBRUSH)GetStockObject(DKGRAY_BRUSH);
    wcex.lpszMenuName = 0;
    wcex.lpszClassName = ATitle;

    RegisterClassEx(&wcex);
}

void Window::initInstance(HINSTANCE hInst, int nCmdShow, const char* ATitle, const char* WTitle, int w, int h) {

    width = w;
    height = h;

    hInstance = hInst;

    int posX = 0;
    int posY = 0;

    DWORD ws = WS_CAPTION | WS_BORDER | WS_MINIMIZEBOX | WS_SYSMENU;
    RECT rc;
    rc.left = posX;
    rc.top = posY;
    rc.right = width;
    rc.bottom = height;
    AdjustWindowRect(&rc, ws, FALSE);

    hWnd = CreateWindowEx( 
        NULL,
        ATitle,
        WTitle,
        ws,
        posX,
        posY,
        rc.right - rc.left + posX,
        rc.bottom - rc.top + posY,
        NULL,
        NULL,
        hInstance,
        NULL
        );

    if( !hWnd ) {
        throw GameException("Window: Could not create Window");
    }

    ShowWindow(hWnd, nCmdShow);
    UpdateWindow(hWnd);
}


const HWND & Window::getHwnd() const {
    return hWnd;
}

const HINSTANCE & Window::get_hInstance() const {
    return hInstance;
}

void Window::setWindowed( BOOL newWindowed ) {
    windowed = newWindowed;
}

BOOL Window::getWindowed() const {
    return windowed;
}

bool Window::toggleWindowed() {
    windowed = !windowed;

    if( windowed ) {
        try {
            g_dx->resetD3D();
        } catch ( const GameException& ) {
            return true;
        }
        //const RECT& o( oldPosition );
        MoveWindow( hWnd, oldPosition.left, oldPosition.top, oldPosition.right-oldPosition.left, oldPosition.bottom-oldPosition.top, TRUE );
    } else {
        // Saves the position of the window
        GetWindowRect( hWnd, &oldPosition );
        try {
            g_dx->resetD3D();
        } catch ( const GameException& ) {
            return true;
        }
    }

    return false;
}

POINT Window::getWindowStartPt()
{
	POINT pt;
	//pt.x = window_startX;
	//pt.y = window_startY;
    pt.x = 0;
    pt.y = 0;
	return pt;
}
