/**
 * CG3 Skeletal Animation Project
 * Jason Winnebeck
 * May 12, 2003
 */

#ifndef _WINDOW_H_
#define _WINDOW_H_

/**
 * Contains data about the game's window and has functions
 * for initializing and controlling the window.
 */
class Window {

public:

     /**
      * Constructor
      *
      */
     Window();

     /**
      * Destructor
      *
      */
    ~Window();

public:

     /**
      * Registers the class with Windows
      *
      * @param  hInst           instance of application
      * @param  ATitle          title of the application
      */
    void registerClass( HINSTANCE hInst, const char* ATitle );

     /**
      * Initializes the window.  Throws GameException if it fails.
      *
      * @param  hInst           instance of application
      * @param  nCmdShow        whether to show the window or not
      * @param  ATitle          title of the application
      * @param  WTitle          title of the window
      * @param  w               width of window in pixels
      * @param  h               height of window in pixels
      */
    void initInstance( HINSTANCE hInst, int nCmdShow, const char* ATitle, const char* WTitle, int w, int h );

    /**
     * Returns the width of the window.
     */
    int getWidth() const {
        return width;
    }

    /**
     * Returns the height of the window.
     */
    int getHeight() const {
        return height;
    }

     /**
      * Returns a reference to hWnd
      *
      * @returns                a reference to hWnd
      */
    const HWND & getHwnd() const;

     /**
      * Returns a refrence to hInstance
      *
      * @returns                a reference to hInstance
      */
    const HINSTANCE & get_hInstance() const;

     /**
      * Set whether window is in windowed mode or not
      *
      * @param  newWindowed     whether the window is in windowed mode or not
      */
    void setWindowed( BOOL newWindowed );

     /**
      * Returns TRUE if the window is in windowed mode
      *
      * @returns                TRUE if the window is in windowed mode
      */
    BOOL getWindowed() const;

    /**
     * If the Window is in windowed mode, go to fullscreen, else go back to
     * windowed mode.
     *
     * @returns     true on error
     */
    bool toggleWindowed();

    /*
    * Gets the window start point
    *
    * @returns window start point
    */
    POINT getWindowStartPt();

private:

    int width;
    int height;

    /**
     * The last position of the window
     *
     */
    RECT oldPosition;

    /**
     * TRUE if the window is in windowed mode
     *
     */
    BOOL windowed;

    /**
     * hWnd
     *
     */
    HWND hWnd;

    /**
     * hInstance
     *
     */
    HINSTANCE hInstance;

};

#endif