/**
 * CG3 Skeletal Animation Project
 * Jason Winnebeck
 * May 12, 2003
 */

#ifndef GLOBAL_H_KLR402
#define GLOBAL_H_KLR402

#define WIN32_LEAN_AND_MEAN
#define _WIN32_WINNT   0x0500
#define _WIN32_WINDOWS 0x0410
#define WINVER         0x0400
#include <windows.h>

#define DIRECTINPUT_VERSION 0x0800

#ifdef _DEBUG
#define TRACE(x)                doTrace( __FILE__, __LINE__, x )
#define TRACE1(x, y)            doTrace( __FILE__, __LINE__, x, y )
#define TRACE2(x, y, z)         doTrace( __FILE__, __LINE__, x, y, z )
#define TRACE3(x, y, z, a)      doTrace( __FILE__, __LINE__, x, y, z, a )
#define TRACE4(x, y, z, a, b)   doTrace( __FILE__, __LINE__, x, y, z, a, b )
#else
#define TRACE(x)                ((void)0)
#define TRACE1(x, y)            ((void)0)
#define TRACE2(x, y, z)         ((void)0)
#define TRACE3(x, y, z, a)      ((void)0)
#define TRACE4(x, y, z, a, b)   ((void)0)
#endif

#define sq(x) ( (x) * (x) )

#define SAFERELEASE( x ) if (x) x->Release(); x=NULL;

#include <d3d8.h>   // directX8 Header File
#include <d3dx8.h>  // directX8 Helper Functions
#include <dxfile.h>
#include <rmxfguid.h> //X File GUIDs
#include <Dxerr8.h>
#include <dinput.h> // DirectInput
#include <assert.h>
#include <ctime>
#include <cmath>
#include <fstream>
#include <sstream>
#include <string>
#include <vector>
#include <list>
#include <algorithm>
#include <set>
#include <map>
#include <functional>
#include <limits>

#ifndef M_PI
#define M_PI       3.14159265358979323846
#endif

#ifndef M_PI_F
#define M_PI_F     3.14159265358979323846f
#endif

//Small utility functions
inline float degToRad( float degrees ) {
    return degrees / 180.0f * M_PI_F;
}

/**
 * Converts any type supporting the ostream operator << to a string.
 */
template <class ObjectType>
std::string toString( const ObjectType& x ) {
    ostringstream o;
    o << x;
    return o.str();
}

/**
 * Trims whitespace from a string.
 */
void trimString( std::string& );

// Typedefs
typedef LPDIRECT3DTEXTURE8 Texture;
typedef LPDIRECT3DDEVICE8 GraphicsDevice;
typedef D3DXMATRIX Matrix;
typedef D3DXQUATERNION Quaternion;
typedef D3DXVECTOR3 Vector;
typedef D3DXVECTOR3 Point;
typedef std::set<std::string> BoneNameSet;

//Global types and variables
#include "GameDX.h"
#include "Window.h"
#include "Timer.h"
#include "GameDXInput.h"
#include "GameException.h"

/**
 * Global pointer to the DirectX stuff
 *
 */
extern GameDX* g_dx;

/**
 * Global pointer to the GameDXInput
 *
 */
extern GameDXInput* g_DI;

/**
 * Global pointer to the Window
 *
 */
extern Window* g_win;

class ConfigFile;

/**
 * Global configuration.
 */
extern ConfigFile* g_cfg;

/**
 * WndProc
 *
 */
LRESULT CALLBACK GlobalWndProc( HWND hWnd, UINT message, 
                                WPARAM wParam, LPARAM lParam );

/**
 * Outputs to the debug window.  Also outputs the filename and line number
 * where TRACE is called.  Works like printf.
 *
 */
void doTrace( const char* fn, int lineno, const char* msg, ... );

#endif
